@@ -242,9 +242,8 @@ static gboolean view_set_zoom_level_at (ChamplainView *view,
 static void tile_state_notify (ChamplainTile *tile,
     G_GNUC_UNUSED GParamSpec *pspec,
     ChamplainView *view);
-static gboolean kinetic_scroll_key_press_cb (ClutterActor *actor,
-    ClutterKeyEvent *event,
-    ChamplainView *view);
+static gboolean kinetic_scroll_key_press_cb (ChamplainView *view,
+    ClutterKeyEvent *event);
 static void champlain_view_go_to_with_duration (ChamplainView *view,
     gdouble latitude,
     gdouble longitude,
@@ -387,7 +386,22 @@ scroll_event (G_GNUC_UNUSED ClutterActor *actor,
     zoom_level = priv->zoom_level + 1;
   else if (event->direction == CLUTTER_SCROLL_DOWN)
     zoom_level = priv->zoom_level - 1;
-    
+  else if (event->direction == CLUTTER_SCROLL_SMOOTH)
+    {
+      gdouble dx, dy;
+
+      clutter_event_get_scroll_delta ((ClutterEvent *)event, &dx, &dy);
+
+      /* Use a threshhold value to avoid jitter */
+      if (fabs(dy) < 0.75)
+        return FALSE;
+
+      if (dy > 0)
+        zoom_level = priv->zoom_level - 1;
+      else
+        zoom_level = priv->zoom_level + 1;
+  }
+
   return view_set_zoom_level_at (view, zoom_level, TRUE, event->x, event->y);
 }
 
@@ -976,7 +990,7 @@ champlain_view_class_init (ChamplainViewClass *champlainViewClass)
    * The #ChamplainView::animation-completed signal is emitted when any animation in the view
    * ends.  This is a detailed signal.  For example, if you want to be signaled
    * only for go-to animation, you should connect to
-   * "animation-completed::go-to".
+   * "animation-completed::go-to". And for zoom, connect to "animation-completed::zoom".
    *
    * Since: 0.4
    */
@@ -1179,13 +1193,13 @@ champlain_view_init (ChamplainView *view)
       G_CALLBACK (panning_completed), view);
   g_signal_connect (priv->kinetic_scroll, "button-press-event",
       G_CALLBACK (kinetic_scroll_button_press_cb), view);
-  g_signal_connect (priv->kinetic_scroll, "key-press-event",
-      G_CALLBACK (kinetic_scroll_key_press_cb), view);
 
   /* Setup stage */
   clutter_actor_add_child (CLUTTER_ACTOR (view), priv->kinetic_scroll);
   priv->zoom_overlay_actor = clutter_actor_new ();
   clutter_actor_add_child (CLUTTER_ACTOR (view), priv->zoom_overlay_actor);
+  g_signal_connect (view, "key-press-event",
+                    G_CALLBACK (kinetic_scroll_key_press_cb), NULL);
 
   /* Setup license */
   priv->license_actor = champlain_license_new ();
@@ -1288,9 +1302,8 @@ champlain_view_scroll (ChamplainView *view,
 
 
 static gboolean
-kinetic_scroll_key_press_cb (G_GNUC_UNUSED ClutterActor *actor,
-    ClutterKeyEvent *event,
-    ChamplainView *view)
+kinetic_scroll_key_press_cb (ChamplainView *view,
+    ClutterKeyEvent *event)
 {
   DEBUG_LOG ()
 
@@ -2538,6 +2551,7 @@ zoom_animation_completed (ClutterActor *actor,
   clutter_actor_show (priv->user_layers);
 
   g_signal_handlers_disconnect_by_func (actor, zoom_animation_completed, view);
+  g_signal_emit_by_name (view, "animation-completed::zoom", NULL);
 }
 
 
